<?php
include_once 'Lib/Logger.inc';

// Dump an array in a pretty format...
function dumpArray($elements) {
    $result = "<ol>\n";
    foreach ($elements as $key => $value) {
        if (is_array($value)) {
            $result .= "<li>Key <b>$key</b> is an array containing:\n" . dumpArray($value) . '</li>';
        } else {
            $value = nl2br(htmlspecialchars($value));
            $result .= "<li>Key <b>$key</b> has value <b>$value</b></li>\n";
        }
    }
    return $result . "</ol>\n";
}

// Function to safely fetch a value from an array, supplying a default
// if no key is present.
function safeGet($arr, $key, $default=false) {
    if ($arr && isset($arr[$key])) {
        $value = $arr[$key];
        if (!is_array($value)) {
            $value = htmlspecialchars(trim($arr[$key]));
        }
        if ($value) {
            return $value;
        }
    }
    return $default;
}

function safeParam($arr, $key, $default = false) {
    if ($arr && isset($arr[$key])) {
		$value = $arr[$key];
		if ( !is_array($value)) {
			$value = htmlentities($arr[$key]);
		}
		if ($value) {
			return $value;
		}
    }
    return $default;
}

function markdown($str) {
    $str = preg_replace('/\*\*(.+)\*\*/u', '<b>$1</b>', $str);
    $str = preg_replace('/\*([^\*]+)\*/u', '<i>$1</i>', $str);
    $str = preg_replace('/#### ([^\n]*)\n/', "<h4>$1</h4>\n", $str);
    $str = preg_replace('/### ([^\n]*)\n/', "<h3>$1</h3>\n", $str);
    $str = preg_replace('/## ([^\n]*)\n/', "<h2>$1</h2>\n", $str);
    $str = preg_replace('/# ([^\n]*)\n/', "<h1>$1</h1>\n", $str);
    $str = preg_replace('/\[([^\]]+)\]\(([^\)]+)\)/', '<a href=\'$2\'>$1</a>', $str);
    $str = preg_replace('/([^\n\r]{2,})(?:(\r\n){2,}|\r{2,}|\n{2,}|$)/u', "<p>$1</p>\n\n", $str);
    return $str;
}

function ensureLoggedInUserIs($id) {
    $userId = safeParam($_SESSION, 'userid');
    if ($id !== $userId) {
        throw new Exception("You are not the owner of that record and don't have edit permission");
    }
}

function currentUser() {
    Logger::instance() -> lib('Util:currentUser called');
    $inSession = session_id();
    if (!empty($inSession)) {
        if (isset($_SESSION['userid'])) {
			$session_id = $_SESSION['userid'];
			Logger::instance() -> lib("currentUser: $session_id");
            return $_SESSION['userid'];
        }
    }
    return 0;
}

function isLoggedIn() {
    Logger::instance() -> lib('Util:isLoggedIn called');
    return currentUser() > 0;
}

function ensureLoggedIn() {
    Logger::instance() -> lib('Util:ensureLoggedIn called');
    if (!isLoggedIn()) {
        $_SESSION['redirect'] = $_SERVER['REQUEST_URI'];
        Logger::instance() -> lib('Util:ensureLoggedIn Failed');
        redirectRelative('auth/login');
        exit();
    }
}

function debug($something) {
    echo "<div class='debug'>\n";
    print_r($something);
    echo "\n</div>\n";
}

function redirect($url) {
    Logger::instance() -> lib("Util:redirect to $url");
    session_write_close();
    header("Location: $url");
    exit();
}

function redirectRelative($url) {
    Logger::instance() -> lib("Util:redirectRelative to $url");
    redirect(relativeURL($url));
}

function relativeUrl($url) {
    $requestURI = explode('/', $_SERVER['REQUEST_URI']);
    $scriptName = explode('/', $_SERVER['SCRIPT_NAME']);

    $dir = array();
    for ($i = 0; $i < sizeof($scriptName); $i++) {
        if ($requestURI[$i] == $scriptName[$i]) {
            $dir[] = $requestURI[$i];
        } else {
            break;
        }
    }
    return implode('/', $dir) . '/' . $url;
}

function __importTemplate($matches) {
    $fileName = trim($matches[1]);
    Logger::instance() -> lib("Util:__importTemplate file:[$fileName]");
    if (!file_exists($fileName)) {
        Logger::instance() -> lib("Util:__importTemplate file:[$fileName] does not exist");
        die("Template $fileName doesn't exist.");
    }
    $contents = file_get_contents($fileName);
    $contents = preg_replace_callback('/%%[ \t]*(.*)[ \t]*%%/', '__importTemplate', $contents);
    return $contents;
}

function __resolveRelativeUrls($matches) {
    return relativeUrl($matches[1]);
}

function __cacheName($view) {
    $cachePath = explode('/', $view);
    $idx = sizeof($cachePath) - 1;
    $cachePath[$idx] = 'cache_' . $cachePath[$idx];
    return implode('/', $cachePath);
}

function renderTemplate($view, $params) {
    Logger::instance() -> lib("Util:renderTemplate $view called");
    if (!file_exists($view)) {
        Logger::instance() -> lib("Util:renderTemplate $view does not exist.");
        # SAS - this should throw an Exception!
        die("File $view doesn't exist.");
    }
    # do we have a cached version?
    clearstatcache();
    $cacheName = __cacheName($view);
    if (file_exists($cacheName) && (filemtime($cacheName) >= filemtime($view))) {
        Logger::instance() -> lib("Util:renderTemplate using cached file: $cacheName");
        $contents = file_get_contents($cacheName);
    } else {
        Logger::instance() -> lib('Util:renderTemplate not cached');
        # we need to build the file (doesn't exist or template is newer)
        $contents = __importTemplate(array('unused', $view));

        $contents = preg_replace_callback('/@@\s*(.*)\s*@@/U', '__resolveRelativeUrls', $contents);

        $patterns = array(
            array('src' => '/{{/', 'dst' => '<?php echo('),
            array('src' => '/}}/', 'dst' => ');?>'),
            array('src' => '/\[\[/', 'dst' => '<?php '),
            array('src' => '/\]\]/', 'dst' => '?>')
        );
        foreach ($patterns as $pattern) {
            $contents = preg_replace($pattern['src'], $pattern['dst'], $contents);
        }
        Logger::instance() -> lib("Util:renderTemplate saving cache file: $cacheName");
        file_put_contents($cacheName, $contents);
    }
    extract($params);
    ob_start();
    eval('?>' . $contents);
    $result = ob_get_contents();
    ob_end_clean();
    echo $result;
}

function error_render($page, $values) {
    // This function takes a page to re-render, and an array of values to pass.
    Logger::instance() -> lib('Util:error_render called');
    renderTemplate(
        "views/$page.inc",
        $values
    );
    exit();
}
